///////////////////////////////////////////////////////////////////////////
//
// Purpose: Generate STL file for "Drink coaster with geometric design"
//
// App:     OpenSCAD (www.openscad.org)
// Author:  Dave Richards, Sunnyvale California USA
// Date:    2015-03-30
// License: Creative Commons - Attribution - Share Alike licence (CC BY-SA)
//
///////////////////////////////////////////////////////////////////////////

// User-specified constants.
outer_diameter=100;
distance_between_dual_outer_rings=5;
inner_diameter=outer_diameter-(distance_between_dual_outer_rings*2);
baseplate_height = 1.3;
trace_thickness=1;

// Do not alter the assignments below.
trace_radius = trace_thickness / 2;
or=outer_diameter/2;
ir=inner_diameter/2;
total_height=baseplate_height + trace_radius;
1u = inner_diameter/6;
2u = 2 * 1u;
3u = 3 * 1u;
4u = 4 * 1u;
$fn=48;// Set 6 or 12 for compilation speed. 24 or 48 for high resolution.
		// Because STLs are composed of interlocking triangles,
		// the most symmetric geometries can be obtained by setting
		// $fn as multiples of 3 (e.g. 3, 6, 9, 12, 15, ...)

module base_coaster() {
	//
	// Creates base coaster without geometric pattern.
	//

	// Base plate.
	cylinder(r=or, h=baseplate_height);

	// Outermost large ring.
	one_ring(y=0, radius=or-trace_radius, minor_radius=trace_radius);

	// Innermost large ring.
	one_ring(y=0, radius=ir, minor_radius=trace_radius);
}
module create_coaster() {
	//
	// Creates entire coaster.
	//
	base_coaster();
	applied_design();
}
module applied_design() {
	difference() {
		create_inner_hexagon_of_rings();
		remove_outer_artifacts();
	}
}
module create_inner_hexagon_of_rings() {
	//
	// Create design with multiple rings having centers equidistant
	// from each other at 60 degree angles, but only so far
	// from the center such that (x^2 + y^2) <= r^2
	//
	one_row(4, -3u * sin(60)); // Far left side row (4 circles).
	one_row(5, -2u * sin(60));
	one_row(6, -1u * sin(60));
	one_row(7,   0 * sin(60)); // Middle row (the longest at 7 circles).
	one_row(6, +1u * sin(60));
	one_row(5, +2u * sin(60));
	one_row(4, +3u * sin(60)); // Far right side row (4 circles).
}
module one_ring(y=0, radius=1u, minor_radius=trace_radius) {
	//
	// Creates one ring on the y axis at position x=0, y=y.
	//
	translate([0, y, baseplate_height])
	rotate_extrude(convexity=10)
	translate([radius, 0, 0])
	// Do not change $fn=6 here.
	// It seems to break manifold for some reason.
	circle(r = minor_radius, $fn=6);
}
module one_row(num_rings, x) {
	//
	// Creates a row of rings.
	//
	y = -((num_rings+1)*1u)/2; // Reposition using one half the length of row.
	translate([x, y, 0])
	union() {
		for (ty = [num_rings:-1:1]) {
			one_ring(y=(ty * 1u), radius=1u, minor_radius=trace_radius);
		}
	}
}
module remove_outer_artifacts() {
	//
	// Remove any leftover artifacts outside of the inner diameter.
	//
	difference() {
		cylinder(r=outer_diameter, h=total_height, $fn=6);
		cylinder(r=ir, h=total_height, $fn=48);
	}
}


// Primary module.
create_coaster();


// Secondary  modules.
//
// base_coaster();
// applied_design();
// create_inner_hexagon_of_rings();
// remove_outer_artifacts();
// one_ring(y=0, radius=1u, minor_radius=trace_radius);
// one_row(7, 0);

